@@ -49,6 +49,7 @@
 #include "../ipc.h"
 #include "../xlog.h"
 #include "../cfg_reload.h"
+#include "../status_report.h"
 #include "mi.h"
 #include "mi_trace.h"
 
@@ -137,13 +138,11 @@ static mi_response_t *mi_version_1(const mi_params_t *params,
 		return 0;
 	}
 
-#ifdef VERSIONTYPE
 	if (add_mi_string(resp_obj, MI_SSTR(VERSIONTYPE), MI_SSTR(THISREVISION))<0) {
 		LM_ERR("failed to add mi item\n");
 		free_mi_response(resp);
 		return 0;
 	}
-#endif
 
 	return resp;
 }
@@ -476,7 +475,95 @@ static mi_response_t *w_xlog_level_1(const mi_params_t *params,
 	return resp;
 }
 
+static mi_response_t *w_log_level_filter_1(const mi_params_t *params,
+								struct mi_handler *async_hdl)
+{
+	mi_response_t *resp;
+	mi_item_t *resp_obj;
+	str consumer;
+	int level_filter;
 
+	if (get_mi_string_param(params, "consumer", &consumer.s, &consumer.len) < 0)
+		return init_mi_param_error();
+
+	if (get_log_consumer_level_filter(&consumer, &level_filter) < 0)
+		return init_mi_error(404, MI_SSTR("Unknown log consumer"));
+
+	resp = init_mi_result_object(&resp_obj);
+	if (!resp)
+		return 0;
+
+	if (add_mi_number(resp_obj, MI_SSTR("Log level filter"), level_filter) < 0)
+		goto error;
+
+	return resp;
+error:
+	free_mi_response(resp);
+	return 0;
+}
+
+static mi_response_t *w_log_level_filter_2(const mi_params_t *params,
+								struct mi_handler *async_hdl)
+{
+	str consumer;
+	int level_filter;
+
+	if (get_mi_string_param(params, "consumer", &consumer.s, &consumer.len) < 0)
+		return init_mi_param_error();
+
+	if (get_mi_int_param(params, "level_filter", &level_filter) < 0)
+		return init_mi_param_error();
+
+	if (set_log_consumer_level_filter(&consumer, level_filter))
+		return init_mi_error(404, MI_SSTR("Unknown log consumer"));
+
+	return init_mi_result_ok();
+}
+
+static mi_response_t *w_log_mute_state_1(const mi_params_t *params,
+								struct mi_handler *async_hdl)
+{
+	mi_response_t *resp;
+	mi_item_t *resp_obj;
+	str consumer;
+	int mute_state;
+
+	if (get_mi_string_param(params, "consumer", &consumer.s, &consumer.len) < 0)
+		return init_mi_param_error();
+
+	if (get_log_consumer_mute_state(&consumer, &mute_state) < 0)
+		return init_mi_error(404, MI_SSTR("Unknown log consumer"));
+
+	resp = init_mi_result_object(&resp_obj);
+	if (!resp)
+		return 0;
+
+	if (add_mi_number(resp_obj, MI_SSTR("mute state"), mute_state) < 0)
+		goto error;
+
+	return resp;
+error:
+	free_mi_response(resp);
+	return 0;
+}
+
+static mi_response_t *w_log_mute_state_2(const mi_params_t *params,
+								struct mi_handler *async_hdl)
+{
+	str consumer;
+	int mute_state;
+
+	if (get_mi_string_param(params, "consumer", &consumer.s, &consumer.len) < 0)
+		return init_mi_param_error();
+
+	if (get_mi_int_param(params, "mute_state", &mute_state) < 0)
+		return init_mi_param_error();
+
+	if (set_log_consumer_mute_state(&consumer, mute_state))
+		return init_mi_error(404, MI_SSTR("Unknown log consumer"));
+
+	return init_mi_result_ok();
+}
 
 static mi_response_t *mi_cachestore(const 	mi_params_t *params, unsigned int expire)
 {
@@ -763,7 +850,7 @@ static mi_response_t *w_reload_routes(const mi_params_t *params,
 
 
 
-static mi_export_t mi_core_cmds[] = {
+static const mi_export_t mi_core_cmds[] = {
 	{ "uptime", "prints various time information about OpenSIPS - "
 		"when it started to run, for how long it runs", 0, init_mi_uptime, {
 		{mi_uptime, {0}},
@@ -817,6 +904,20 @@ static mi_export_t mi_core_cmds[] = {
 		{EMPTY_MI_RECIPE}
 		}
 	},
+	{ "log_level_filter", "gets/sets the per consumer log level filter",
+		0, 0, {
+		{w_log_level_filter_1, {"consumer", 0}},
+		{w_log_level_filter_2, {"consumer", "level_filter", 0}},
+		{EMPTY_MI_RECIPE}
+		}
+	},
+	{ "log_mute_state", "mute/unmute a log consumer",
+		0, 0, {
+		{w_log_mute_state_1, {"consumer", 0}},
+		{w_log_mute_state_2, {"consumer", "mute_state", 0}},
+		{EMPTY_MI_RECIPE}
+		}
+	},
 
 #if defined(Q_MALLOC) && defined(DBG_MALLOC)
 	{ "shm_check", "complete scan of the shared memory pool "
@@ -897,6 +998,36 @@ static mi_export_t mi_core_cmds[] = {
 		{EMPTY_MI_RECIPE}
 		}
 	},
+	{ "sr_get_status", "gets the status (only) of a 'status-report' "
+	"group/identifier", 0, 0, {
+		{mi_sr_get_status, {"group",0}},
+		{mi_sr_get_status, {"group","identifier",0}},
+		{EMPTY_MI_RECIPE}
+		}
+	},
+	{ "sr_list_status", "list the status of all the identifiers in OpenSIPS"
+	" or from a certain 'status-report' group", 0, 0, {
+		{mi_sr_list_status, {0}},
+		{mi_sr_list_status, {"group",0}},
+		{EMPTY_MI_RECIPE}
+		}
+	},
+	{ "sr_list_reports", "list the reports produced by some 'status-report' "
+	"identifiers / groups" , 0, 0, {
+		{mi_sr_list_reports, {0}},
+		{mi_sr_list_reports, {"group",0}},
+		{mi_sr_list_reports, {"group","identifier",0}},
+		{EMPTY_MI_RECIPE}
+		}
+	},
+	{ "sr_list_identifiers", "list the identifiers from a group or all",
+	0, 0, {
+		{mi_sr_list_identifiers, {0}},
+		{mi_sr_list_identifiers, {"group",0}},
+		{EMPTY_MI_RECIPE}
+		}
+	},
+
 	{ "help", "prints information about MI commands usage", 0, 0, {
 		{w_mi_help, {0}},
 		{w_mi_help_1, {"mi_cmd", 0}},
