@@ -138,9 +138,12 @@ int cassandra_reopen(cassandra_con *cass_con)
 	return 0;
 }
 
-int cassandra_new_connection(cassandra_con *con, char *host, int port)
+int cassandra_new_connection(cassandra_con *con, char *host, int port, char *username, char *password)
 {
 	con->cluster = cass_cluster_new();
+	if (username && password) {
+	  cass_cluster_set_credentials(con->cluster, username, password);
+	}
 	if (!con->cluster) {
 		LM_ERR("Failed to create Cassandra Cluster object\n");
 		return -1;
@@ -256,7 +259,7 @@ void *cassandra_init_connection(struct cachedb_id *id)
 	con->table = table;
 	con->cnt_table = cnt_table;
 
-	if (cassandra_new_connection(con, id->host, id->port) < 0) {
+	if (cassandra_new_connection(con, id->host, id->port, id->username, id->password) < 0) {
 		LM_ERR("failed to create new connection to Cassandra\n");
 		pkg_free(con);
 		return NULL;
@@ -311,7 +314,7 @@ const CassResult *execute_query(CassSession* session, CassStatement *statement,
 	start_expire_timer(start, cassandra_exec_threshold);
 	rc = cass_future_wait_timed(query_future, 1000*cassandra_query_timeout);
 	_stop_expire_timer(start, cassandra_exec_threshold, op_name,
-						NULL, 0, 0, cdb_slow_queries, cdb_total_queries);
+						"", 0, 0, cdb_slow_queries, cdb_total_queries);
 	if (rc == 0) {
 		LM_ERR("Cassandra query took too long\n");
 		goto error;
