@@ -102,6 +102,11 @@
 #define COMPRESS_CB (1<<0)
 #define COMPACT_CB (1<<1)
 
+#define NO_COMPACT_FORM (1<<1)
+
+#define CL_NAME_NO_DELIM		"Content-Length"
+#define CL_NAME_NO_DELIM_LEN	(sizeof(CL_NAME_NO_DELIM) - 1)
+
 #define SET_GLOBAL_CTX(pos, value) \
 	(context_put_ptr(CONTEXT_GLOBAL, current_processing_ctx, pos, value))
 
@@ -122,14 +127,15 @@ int tm_compress_ctx_pos, tm_compact_ctx_pos;
 static int fixup_whitelist_compact(void**);
 static int fixup_whitelist_compress(void**);
 static int fixup_whitelist_free(void **);
+static int fixup_mc_compact_flags(void **);
 
-static int mc_compact(struct sip_msg* msg, mc_whitelist_p wh_list);
-static int mc_compact_cb(char** buf, mc_whitelist_p wh_list, int, int*);
+static int mc_compact(struct sip_msg* msg, mc_whitelist_p wh_list, void* flags_p);
+static int mc_compact_cb(char** buf, struct mc_compact_args* mc_compact_args, int, int*);
 
 static int mc_compress(struct sip_msg* msg, int* algo, int* flags,
 		mc_whitelist_p wh_list);
 int mc_compress_cb(char** buf, void* param, int type, int* olen);
-static inline int mc_ndigits(int x);
+static inline unsigned int mc_ndigits(int x);
 static inline void parse_algo_hdr(struct hdr_field* algo_hdr, int* algo, int* b64_required);
 
 static int mc_decompress(struct sip_msg*);
@@ -149,15 +155,17 @@ static str body_in  = {NULL, 0},
 	   hdr_out  = {NULL, 0},
 	   buf_out  = {NULL, 0};
 
-static param_export_t mod_params[]={
+static const param_export_t mod_params[]={
 	{ "compression_level", INT_PARAM, &mc_level},
 	{0,0,0}
 };
 
-static cmd_export_t cmds[]={
+static const cmd_export_t cmds[]={
 	{"mc_compact",	  (cmd_function)mc_compact, {
 		{CMD_PARAM_STR|CMD_PARAM_OPT|CMD_PARAM_FIX_NULL,
 			fixup_whitelist_compact, fixup_whitelist_free},
+		{CMD_PARAM_STR|CMD_PARAM_OPT|CMD_PARAM_FIX_NULL,
+			 fixup_mc_compact_flags, 0},
 		{0, 0, 0}},
 		REQUEST_ROUTE|ONREPLY_ROUTE|LOCAL_ROUTE|FAILURE_ROUTE},
 	{"mc_compress",	  (cmd_function)mc_compress, {
@@ -282,6 +290,7 @@ void wrap_tm_func(struct cell* t, int type, struct tmcb_params* p)
 {
 	int ret = 0;
 	mc_whitelist_p wh_list = NULL;
+	struct mc_compact_args* mc_compact_args = NULL;
 	struct mc_comp_args* args = NULL;
 	char* buf = t->uac[p->code].request.buffer.s;
 	int olen = t->uac[p->code].request.buffer.len;
@@ -301,10 +310,10 @@ void wrap_tm_func(struct cell* t, int type, struct tmcb_params* p)
 
 		case COMPACT_CB:
 			/* if not registered yet we take from global context */
-			if ((wh_list = GET_GLOBAL_CTX(compact_ctx_pos)) == NULL)
+			if ((mc_compact_args = GET_GLOBAL_CTX(compact_ctx_pos)) == NULL)
 				break;
 
-			if ((ret = mc_compact_cb(&buf, wh_list, TM_CB, &olen)) < 0)
+			if ((ret = mc_compact_cb(&buf, mc_compact_args, TM_CB, &olen)) < 0)
 				LM_ERR("compaction failed\n");
 
 			SET_GLOBAL_CTX(compact_ctx_pos, NULL);
@@ -319,6 +328,7 @@ void wrap_tm_func(struct cell* t, int type, struct tmcb_params* p)
 	/* free whitelists for both actions */
 	if (wh_list)
 		free_whitelist(wh_list);
+	free_mc_compact_args(mc_compact_args);
 	if (ret < 0)
 		return;
 
@@ -341,6 +351,7 @@ int wrap_msg_func(str* buf, struct sip_msg* p_msg, int type)
 {
 	int ret = 0;
 	struct mc_comp_args* args;
+	struct mc_compact_args *mc_compact_args = NULL;
 	mc_whitelist_p wh_list = NULL;
 	int olen=buf->len;
 
@@ -363,10 +374,10 @@ int wrap_msg_func(str* buf, struct sip_msg* p_msg, int type)
 		break;
 
 	case COMPACT_CB:
-		if ((wh_list = GET_GLOBAL_CTX(compact_ctx_pos))==NULL)
+		if ((mc_compact_args = GET_GLOBAL_CTX(compact_ctx_pos))==NULL)
 			break;
 
-		if ((ret = mc_compact_cb(&buf->s, wh_list, PROCESSING_CB, &olen)) < 0)
+		if ((ret = mc_compact_cb(&buf->s, mc_compact_args, PROCESSING_CB, &olen)) < 0)
 			LM_ERR("compaction failed\n");
 
 		SET_GLOBAL_CTX(compact_ctx_pos, NULL);
@@ -376,6 +387,7 @@ int wrap_msg_func(str* buf, struct sip_msg* p_msg, int type)
 	/* free whitelists for both actions */
 	if (wh_list)
 		free_whitelist(wh_list);
+	free_mc_compact_args(mc_compact_args);
 	if (ret < 0)
 		return -1;
 
@@ -450,6 +462,28 @@ static int fixup_whitelist_free(void **param)
 	return free_whitelist(*param);
 }
 
+static int fixup_mc_compact_flags(void **param)
+{
+	str *s = (str *) *param;
+	int st;
+	unsigned long flags = 0;
+
+	if (s) {
+		for (st = 0; st < s->len; st++) {
+			switch (s->s[st]) {
+				case 'n':
+					flags |= NO_COMPACT_FORM;
+					break;
+					default:
+						LM_WARN("unknown option `%c'\n", s->s[st]);
+			}
+		}
+	}
+
+	*param = (void *)flags;
+	return 0;
+}
+
 
 /*
  * Memcpy and update length wrapper
@@ -590,14 +624,28 @@ error:
  * 3) Headers which not in whitelist will be removed
  * 4) Unnecessary sdp body codec attributes lower than 96 removed
  */
-static int mc_compact(struct sip_msg* msg, mc_whitelist_p wh_list)
+static int mc_compact(struct sip_msg* msg, mc_whitelist_p wh_list, void* flags_p)
 {
+	struct mc_compact_args *mc_compact_args_p;
+
 	/* first check if anyone else has called mc_compact() on this msg */
 	if (GET_GLOBAL_CTX(compact_ctx_pos))
 		return -1;
 
-	wh_list = mc_dup_whitelist(wh_list);
-	SET_GLOBAL_CTX(compact_ctx_pos, (void*)wh_list);
+	mc_compact_args_p = pkg_malloc(sizeof(struct mc_compact_args));
+	if (mc_compact_args_p==NULL) {
+		LM_ERR("no more pkg mem\n");
+		goto error;
+	}
+
+	mc_compact_args_p->wh_list = mc_dup_whitelist(wh_list);
+	if (mc_compact_args_p->wh_list==NULL) {
+		LM_ERR("no more pkg mem\n");
+		goto error;
+	}
+
+	mc_compact_args_p->flags = (unsigned int)(unsigned long)flags_p;
+	SET_GLOBAL_CTX(compact_ctx_pos, (void*)mc_compact_args_p);
 
 	/* register stateless callbacks */
 	if (register_post_raw_processing_cb(wrap_msg_compact, POST_RAW_PROCESSING, 1/*to be freed*/) < 0) {
@@ -605,7 +653,7 @@ static int mc_compact(struct sip_msg* msg, mc_whitelist_p wh_list)
 		goto error;
 	}
 
-	if (tm_api.t_gett && msg->flags&FL_TM_CB_REGISTERED)
+	if (tm_api.t_gett && msg->msg_flags&FL_TM_CB_REGISTERED)
 		goto error;
 
 	/*register tm callback if tm api */
@@ -613,7 +661,7 @@ static int mc_compact(struct sip_msg* msg, mc_whitelist_p wh_list)
 			tm_api.register_tmcb( msg, 0, TMCB_PRE_SEND_BUFFER,
 				wrap_tm_compact, NULL, 0) != 1) {
 		LM_ERR("failed to add tm TMCB_PRE_SEND_BUFFER callback\n");
-		msg->flags |= FL_TM_CB_REGISTERED;
+		msg->msg_flags |= FL_TM_CB_REGISTERED;
 		goto error;
 	}
 
@@ -623,14 +671,14 @@ static int mc_compact(struct sip_msg* msg, mc_whitelist_p wh_list)
 
 error:
 	SET_GLOBAL_CTX(compact_ctx_pos, NULL);
-	free_whitelist(wh_list);
+	free_mc_compact_args(mc_compact_args_p);
 	return -1;
 }
 
 /*
  *
  */
-static int mc_compact_cb(char** buf_p, mc_whitelist_p wh_list, int type, int* olen)
+static int mc_compact_cb(char** buf_p, struct mc_compact_args *mc_compact_args, int type, int* olen)
 {
 	int i;
 	int msg_total_len;
@@ -684,7 +732,7 @@ static int mc_compact_cb(char** buf_p, mc_whitelist_p wh_list, int type, int* ol
 			break;
 		}
 
-		if (mc_is_in_whitelist(hf, wh_list)) {
+		if (mc_is_in_whitelist(hf, mc_compact_args->wh_list)) {
 			if (hdr_mask[hf->type]) {
 				/* If hdr already found or hdr of type other */
 				if (append_hf2lst(&hdr_mask[hf->type], hf,
@@ -699,6 +747,7 @@ static int mc_compact_cb(char** buf_p, mc_whitelist_p wh_list, int type, int* ol
 
 				/* Get the compact form of the header */
 				if (hf->type != HDR_OTHER_T &&
+					!(mc_compact_args->flags & NO_COMPACT_FORM) &&
 					(c=get_compact_form(hf)) != NO_FORM) {
 
 					hf->name.s = &COMPACT_FORMS[c];
@@ -800,8 +849,13 @@ static int mc_compact_cb(char** buf_p, mc_whitelist_p wh_list, int type, int* ol
 	memset(hf, 0, sizeof(struct hdr_field));
 
 	hf->type = HDR_CONTENTLENGTH_T;
-	hf->name.s = &COMPACT_FORMS[get_compact_form(hf)];
-	hf->name.len = 1;
+	if (mc_compact_args->flags & NO_COMPACT_FORM) {
+		hf->name.s = CL_NAME_NO_DELIM;
+		hf->name.len = CL_NAME_NO_DELIM_LEN;
+	} else {
+		hf->name.s = &COMPACT_FORMS[get_compact_form(hf)];
+		hf->name.len = 1;
+	}
 
 	if (new_body_len <= CRLF_LEN)
 		new_body_len = 0;
@@ -846,7 +900,8 @@ again:
 		if (hdr_mask[i]) {
 			/* Compact form name so the header have
 				to be built */
-			if (LOWER_CASE(hdr_mask[i]->name.s)) {
+			if (LOWER_CASE(hdr_mask[i]->name.s) ||
+				hdr_mask[i]->type == HDR_CONTENTLENGTH_T) {
 				/* Copy the name of the header */
 				wrap_copy_and_update(&new_buf.s,
 					hdr_mask[i]->name.s,
@@ -957,21 +1012,24 @@ again:
 memerr:
 	LM_ERR("No more pkg mem\n");
 free_mem:
-	free_hdr_mask(hdr_mask);
+	if (hdr_mask != NULL)
+		free_hdr_mask(hdr_mask);
 	return -1;
 }
 
 /*
  *
  */
-static inline int mc_ndigits(int x)
+static inline unsigned int mc_ndigits(int x)
 {
-	if (x == 0)
-		return 1;
+	unsigned int n = 0;
 
-	if (x > 10)
-		return 1 + mc_ndigits(x/10);
-	else return 1;
+	while (x >= 10) {
+		x /= 10;
+		n++;
+	}
+
+	return n + 1;
 }
 
 /*
@@ -1045,7 +1103,7 @@ static int mc_compress(struct sip_msg* msg, int *algo_p, int *flags_p,
 		goto end;
 	}
 
-	if (tm_api.t_gett && msg->flags&FL_TM_CB_REGISTERED) {
+	if (tm_api.t_gett && msg->msg_flags&FL_TM_CB_REGISTERED) {
 		ret = 1;
 		goto end;
 	}
@@ -1055,7 +1113,7 @@ static int mc_compress(struct sip_msg* msg, int *algo_p, int *flags_p,
 			tm_api.register_tmcb( msg, 0, TMCB_PRE_SEND_BUFFER,
 				wrap_tm_compress, NULL, 0) != 1) {
 		LM_ERR("failed to add tm TMCB_PRE_SEND_BUFFER callback\n");
-		msg->flags |= FL_TM_CB_REGISTERED;
+		msg->msg_flags |= FL_TM_CB_REGISTERED;
 		goto end;
 	}
 
@@ -1676,7 +1734,7 @@ static int mc_decompress(struct sip_msg* msg)
 	/*If compressed with this module there are great chances that Content-Encoding is last*/
 	hdr_vec[3] = msg->last_header;
 
-	if (!is_content_encoding(hdr_vec[3])) {
+	if (hdr_vec[3] && !is_content_encoding(hdr_vec[3])) {
 		hdr_vec[3] = NULL;
 		for (hf = msg->headers; hf; hf = hf->next) {
 			if (is_content_encoding(hf)) {
@@ -1754,7 +1812,7 @@ static int mc_decompress(struct sip_msg* msg)
 
 	b64_required=0;
 	if (hdr_vec[2]) {
-		parse_algo_hdr(hdr_vec[3], &algo, &b64_required);
+		parse_algo_hdr(hdr_vec[2], &hdrs_algo, &b64_required);
 	}
 
 	if (b64_required > 0 &&  hdr_vec[1]) {
