@@ -34,7 +34,7 @@
 #include "../../db/db_ut.h"
 #include "../../db/db_insertq.h"
 #include "../../db/db_res.h"
-#include "my_con.h"
+#include "sqlite_con.h"
 #include "val.h"
 #include "res.h"
 #include "row.h"
@@ -55,6 +55,7 @@ static int db_sqlite_store_result(const db_con_t* _h, db_res_t** _r, const db_va
 static int db_sqlite_bind_values(sqlite3_stmt* stmt, const db_val_t* _v, const int _n);
 #endif
 static int db_sqlite_free_result_internal(const db_con_t* _h, db_res_t* _r);
+static void db_sqlite_free_result_rows(db_res_t* _r);
 
 static int db_sqlite_submit_dummy_query(const db_con_t* _h, const str* _s)
 {
@@ -92,7 +93,7 @@ static inline int db_copy_rest_of_count(const str* _qh, str* count_query)
 	const str searched_str = {" from ", sizeof(" from ")-1};
 
 	count_query->len = sizeof(COUNT_QUERY)-1;
-	if ((found=str_strstr(_qh, &searched_str)) != NULL) {
+	if ((found=str_strcasestr(_qh, &searched_str)) != NULL) {
 		const int len=_qh->len-(found-_qh->s);
 		/* check for overflow */
 		if (len > COUNT_BUF_SIZE-(sizeof(COUNT_QUERY)-1)) {
@@ -116,10 +117,7 @@ db_sqlite_get_query_rows(const db_con_t* _h, const str* query, const db_val_t* _
 	int ret;
 	sqlite3_stmt* stmt;
 
-again:
 	ret=sqlite3_prepare_v2(CON_CONNECTION(_h), query->s, query->len, &stmt, NULL);
-	if (ret == SQLITE_BUSY)
-		goto again;
 
 	if (ret != SQLITE_OK) {
 		LM_ERR("failed to prepare query\n");
@@ -133,11 +131,7 @@ again:
 	}
 #endif
 
-again2:
 	ret=sqlite3_step(stmt);
-	if (ret == SQLITE_BUSY)
-		goto again2;
-
 	if (ret != SQLITE_ROW) {
 		sqlite3_finalize(stmt);
 		LM_ERR("failed to fetch query size\n");
@@ -179,12 +173,9 @@ int db_sqlite_query(const db_con_t* _h, const db_key_t* _k, const db_op_t* _op,
 	}
 
 
-again:
 	ret=sqlite3_prepare_v2(CON_CONNECTION(_h),
 				query_holder.s, query_holder.len, &CON_SQLITE_PS(_h), NULL);
 
-	if (ret==SQLITE_BUSY)
-		goto again;
 	if (ret!=SQLITE_OK)
 		LM_ERR("failed to prepare: (%s)\n", sqlite3_errmsg(CON_CONNECTION(_h)));
 
@@ -203,7 +194,7 @@ again:
 		 * because later won't have the query string */
 		ret = CON_PS_ROWS(_h) = db_sqlite_get_query_rows(_h, &count_str, _v, _n);
 	}
-	if( ret < 0 ){
+	if( ret < 0 && _r ){
 		db_sqlite_free_result_internal(_h,*_r);
 	}
 
@@ -265,10 +256,7 @@ int db_sqlite_fetch_result(const db_con_t* _h, db_res_t** _r, const int nrows)
 		}
 	} else {
 		/* free old rows */
-		if(RES_ROWS(*_r)!=0)
-			db_free_rows(*_r);
-		RES_ROWS(*_r) = 0;
-		RES_ROW_N(*_r) = 0;
+		db_sqlite_free_result_rows(*_r);
 	}
 
 	/* determine the number of rows remaining to be processed */
@@ -385,11 +373,8 @@ int db_sqlite_raw_query(const db_con_t* _h, const str* _s, db_res_t** _r)
 		return -1;
 	}
 
-again:
 	ret=sqlite3_prepare_v2(CON_CONNECTION(_h),
 				_s->s, _s->len, &CON_SQLITE_PS(_h), NULL);
-	if (ret==SQLITE_BUSY)
-		goto again;
 	if (ret!=SQLITE_OK)
 		LM_ERR("failed to prepare: (%s)\n",
 				sqlite3_errmsg(CON_CONNECTION(_h)));
@@ -401,7 +386,7 @@ again:
 		 * because later won't have the query string */
 		ret = CON_PS_ROWS(_h) = db_sqlite_get_query_rows(_h, &count_str, NULL, 0);
 	}
-	if( ret < 0 ){
+	if( ret < 0 && _r ){
 		db_sqlite_free_result_internal(_h,*_r);
 	}
 
@@ -434,11 +419,8 @@ int db_sqlite_insert(const db_con_t* _h, const db_key_t* _k, const db_val_t* _v,
 		return ret;
 	}
 
-again:
 	ret=sqlite3_prepare_v2(CON_CONNECTION(_h),
 			query_holder.s, query_holder.len, &stmt, NULL);
-	if (ret==SQLITE_BUSY)
-		goto again;
 	if (ret!=SQLITE_OK)
 		LM_ERR("failed to prepare: (%s)\n",
 				sqlite3_errmsg(CON_CONNECTION(_h)));
@@ -450,10 +432,7 @@ again:
 	}
 #endif
 
-again2:
 	ret = sqlite3_step(stmt);
-	if (ret==SQLITE_BUSY)
-		goto again2;
 
 	if (ret != SQLITE_DONE) {
 		LM_ERR("insert query failed %s\n", sqlite3_errmsg(CON_CONNECTION(_h)));
@@ -494,11 +473,8 @@ int db_sqlite_delete(const db_con_t* _h, const db_key_t* _k, const db_op_t* _o,
 	}
 
 
-again:
 	ret=sqlite3_prepare_v2(CON_CONNECTION(_h),
 			query_holder.s, query_holder.len, &stmt, NULL);
-	if (ret==SQLITE_BUSY)
-		goto again;
 	if (ret!=SQLITE_OK)
 		LM_ERR("failed to prepare: (%s)\n",
 				sqlite3_errmsg(CON_CONNECTION(_h)));
@@ -510,10 +486,7 @@ again:
 	}
 #endif
 
-again2:
 	ret = sqlite3_step(stmt);
-	if (ret==SQLITE_BUSY)
-		goto again2;
 
 	if (ret != SQLITE_DONE) {
 		LM_ERR("insert query failed %s\n", sqlite3_errmsg(CON_CONNECTION(_h)));
@@ -556,11 +529,8 @@ int db_sqlite_update(const db_con_t* _h, const db_key_t* _k, const db_op_t* _o,
 		return ret;
 	}
 
-again:
 	ret=sqlite3_prepare_v2(CON_CONNECTION(_h),
 			query_holder.s, query_holder.len, &stmt, NULL);
-	if (ret==SQLITE_BUSY)
-		goto again;
 	if (ret!=SQLITE_OK)
 		LM_ERR("failed to prepare: (%s)\n",
 				sqlite3_errmsg(CON_CONNECTION(_h)));
@@ -573,10 +543,7 @@ again:
 	}
 #endif
 
-again2:
 	ret = sqlite3_step(stmt);
-	if (ret==SQLITE_BUSY)
-		goto again2;
 
 	if (ret != SQLITE_DONE) {
 		LM_ERR("insert query failed %s\n", sqlite3_errmsg(CON_CONNECTION(_h)));
@@ -613,11 +580,8 @@ int db_sqlite_replace(const db_con_t* _h, const db_key_t* _k, const db_val_t* _v
 		return ret;
 	}
 
-again:
 	ret=sqlite3_prepare_v2(CON_CONNECTION(_h),
 			query_holder.s, query_holder.len, &stmt, NULL);
-	if (ret==SQLITE_BUSY)
-		goto again;
 	if (ret!=SQLITE_OK)
 		LM_ERR("failed to prepare: (%s)\n",
 				sqlite3_errmsg(CON_CONNECTION(_h)));
@@ -629,10 +593,7 @@ again:
 	}
 #endif
 
-again2:
 	ret = sqlite3_step(stmt);
-	if (ret==SQLITE_BUSY)
-		goto again2;
 
 	if (ret != SQLITE_DONE) {
 		LM_ERR("insert query failed %s\n", sqlite3_errmsg(CON_CONNECTION(_h)));
@@ -706,11 +667,8 @@ int db_last_inserted_id(const db_con_t* _h)
 	sql_str.s = sql_buf;
 	sql_str.len = off;
 
-again:
 	ret=sqlite3_prepare_v2(CON_CONNECTION(_h),
 			sql_str.s, sql_str.len, &stmt, NULL);
-	if (ret==SQLITE_BUSY)
-		goto again;
 	if (ret!=SQLITE_OK)
 		LM_ERR("failed to prepare: (%s)\n",
 				sqlite3_errmsg(CON_CONNECTION(_h)));
@@ -722,10 +680,7 @@ again:
 	}
 #endif
 
-again2:
 	ret = sqlite3_step(stmt);
-	if (ret==SQLITE_BUSY)
-		goto again2;
 
 	if (ret != SQLITE_DONE) {
 		LM_ERR("insert query failed %s\n", sqlite3_errmsg(CON_CONNECTION(_h)));
@@ -824,6 +779,32 @@ int db_sqlite_free_result(db_con_t* _h, db_res_t* _r)
 	return 0;
 }
 
+/**
+ * Release a result set from memory.
+ * \param _r result set whose rows and values should be freed
+ * \return void
+ */
+static void db_sqlite_free_result_rows(db_res_t* _r)
+{
+	db_val_t* values;
+
+	if (!_r) {
+		LM_DBG("nothing to free!\n");
+		return;
+	}
+
+	if(RES_ROWS(_r)!=0)
+	{
+		values = _r->rows[0].values;
+		/* db_sqlite_allocate_rows allocates memory for rows and values separately.
+		 * Hence freeing rows using generic function and then values separately*/
+		db_free_rows(_r);
+		pkg_free(values);
+	}
+	RES_ROWS(_r) = 0;
+	RES_ROW_N(_r) = 0;
+}
+
 /**
  * Retrieve a result set
  * \param _h handle to the database
