@@ -75,7 +75,7 @@ int send_esct(struct sip_msg *msg, str callid_ori, str from_tag){
 	callid.s = callidHeader,
 		callid.len = strlen(callidHeader);
 
-	hash_code= core_hash(&callid, 0, emet_size);
+	hash_code= core_hash(&callid, NULL, emet_size);
 	LM_DBG("********************************************HASH_CODE%d\n", hash_code);
 
 	info_call= search_ehtable(call_htable, callidHeader, ftag, hash_code, 1);
@@ -215,6 +215,11 @@ int treat_parse_esrResponse(struct sip_msg *msg, ESCT *call_cell, PARSED *parsed
 		call_cell->ert_srid = "";
 
 		char *r = strstr(call_cell->esgwri, "@");
+		if (!r) {
+			LM_ERR("String '@' not found\n");
+			return -1;
+		}
+
 		r++;
 		int tam_esgw = call_cell->esgwri + strlen(call_cell->esgwri) - r;
 
