@@ -50,10 +50,6 @@ int rmq_send(rmq_send_t * rmqs);
 void rmq_free_param(rmq_params_t *rmqp);
 void rmq_destroy(evi_reply_sock *sock);
 
-extern struct timeval conn_timeout_tv;
-#if defined AMQP_VERSION && AMQP_VERSION >= 0x00090000
-extern struct timeval rpc_timeout_tv;
-#endif
 extern str rmq_static_holder;
 
 #endif
