@@ -46,21 +46,34 @@
 #include "../../str.h"
 #include "../../ut.h"
 #include "../../lib/sliblist.h"
+#include "../../reactor_proc.h"
 #include "httpd_load.h"
+#include "httpd_proc.h"
 
 
 extern int port;
 extern str ip;
 extern str buffer;
+extern unsigned int hd_conn_timeout_s;
 extern int post_buf_size;
+extern str tls_cert_file;
+extern str tls_key_file;
+extern str tls_ciphers;
 extern struct httpd_cb *httpd_cb_list;
 static union sockaddr_union httpd_server_info;
 
+
+extern int receive_buf_size;
+extern char *httpd_receive_buff;
+extern int httpd_receive_buff_pos;
+
 static const str MI_HTTP_U_URL = str_init("<html><body>"
 "Unable to parse URL!</body></html>");
 static const str MI_HTTP_U_METHOD = str_init("<html><body>"
 "Unsupported HTTP request!</body></html>");
 
+static char * load_file(char * );
+
 /**
  * Data structure to store inside elents of slinkedl_list list.
  */
@@ -124,6 +137,26 @@ int httpd_get_val(void *e_data, void *data, void *r_data)
 	return 0;
 }
 
+int httpd_merge_data(void *e_data, void *data, void *r_data)
+{
+	str_str_t *kv = (str_str_t*)e_data;
+	if (kv==NULL) {
+		LM_ERR("null data\n");
+	} else {
+		LM_DBG("data=[%p] [%p][%p] [%.*s]->[%.*s]\n",
+			kv, kv->key.s, kv->val.s,
+			kv->key.len, kv->key.s,
+			kv->val.len, kv->val.s);
+		if (httpd_receive_buff_pos + kv->val.len > receive_buf_size) {
+			LM_ERR("Received too big HTTP request ( %d bytes ), increase receive_buf_size param value\n", httpd_receive_buff_pos + kv->val.len);
+		} else {
+			memcpy(httpd_receive_buff+httpd_receive_buff_pos,kv->val.s,kv->val.len);
+			httpd_receive_buff_pos+=kv->val.len;
+		}
+	}
+	return 0;
+}
+
 /**
  * Function to print data stored in  slinkedl_list list elemnts.
  * For debugging purposes only.
@@ -191,7 +224,7 @@ skip:
  * Handle regular POST data.
  *
  */
-static int post_iterator (void *cls,
+static MHD_RET post_iterator (void *cls,
 		enum MHD_ValueKind kind,
 		const char *key,
 		const char *filename,
@@ -282,7 +315,7 @@ static int post_iterator (void *cls,
  * @return MHD_YES to continue iterating,
  *         MHD_NO to abort the iteration.
  */
-int getConnectionHeader(void *cls, enum MHD_ValueKind kind,
+MHD_RET getConnectionHeader(void *cls, enum MHD_ValueKind kind,
 					const char *key, const char *value)
 {
 	struct post_request *pr = (struct post_request*)cls;
@@ -323,6 +356,8 @@ int getConnectionHeader(void *cls, enum MHD_ValueKind kind,
 			pr->content_type = HTTPD_APPLICATION_JSON_CNT_TYPE;
 		else if (strncasecmp("text/html", value, 9) == 0)
 			pr->content_type = HTTPD_TEXT_HTML_TYPE;
+		else if (strncasecmp("text/plain", value, 10) == 0)
+			pr->content_type = HTTPD_TEXT_PLAIN_TYPE;
 		else {
 			pr->content_type = HTTPD_UNKNOWN_CNT_TYPE;
 			LM_ERR("Unexpected Content-Type=[%s]\n", value);
@@ -398,8 +433,8 @@ union sockaddr_union* httpd_get_server_info(void)
 	return &httpd_server_info;
 }
 
-
-int answer_to_connection (void *cls, struct MHD_Connection *connection,
+/* 0x00097001 changed the returned result */
+MHD_RET answer_to_connection (void *cls, struct MHD_Connection *connection,
 		const char *url, const char *method,
 		const char *version, const char *upload_data,
 		size_t *upload_data_size, void **con_cls)
@@ -444,11 +479,14 @@ int answer_to_connection (void *cls, struct MHD_Connection *connection,
 	cl_socket = (union sockaddr_union *)MHD_get_connection_info(connection,
 			MHD_CONNECTION_INFO_CLIENT_ADDRESS)->client_addr;
 
+	if (!MHD_set_connection_option(connection, MHD_CONNECTION_OPTION_TIMEOUT, hd_conn_timeout_s))
+		LM_ERR("failed to set connection timeout\n");
+
 #if ( MHD_VERSION >= 0x000092800 )
 	sv_sockfd = MHD_get_connection_info(connection, MHD_CONNECTION_INFO_CONNECTION_FD)->connect_fd;
 	if (getsockname( sv_sockfd, &httpd_server_info.s, &addrlen) < 0) {
 		LM_ERR("cannot resolve server's IP: %s:%d\n", strerror(errno), errno);
-		return -1;
+		return MHD_NO;
 	}
 
 	/* we could do
@@ -485,9 +523,12 @@ int answer_to_connection (void *cls, struct MHD_Connection *connection,
 					if (cb) {
 						normalised_url = &url[cb->http_root->len+1];
 						LM_DBG("normalised_url=[%s]\n", normalised_url);
-						kv = slinkedl_peek(pr->p_list);
-						if (kv)
-							saved_body = ((str_str_t *)kv)->val;
+						httpd_receive_buff_pos=0;
+						slinkedl_traverse(pr->p_list,&httpd_merge_data, NULL, NULL);
+						if (httpd_receive_buff_pos>0) {
+							saved_body.len = httpd_receive_buff_pos;
+							saved_body.s = httpd_receive_buff;
+						}
 						ret_code = cb->callback(cls, (void*)connection,
 								normalised_url,
 								method, version,
@@ -516,12 +557,7 @@ int answer_to_connection (void *cls, struct MHD_Connection *connection,
 					err log printed in getConnectionHeader() */
 					return MHD_NO;
 				}
-				if (*upload_data_size != pr->content_len) {
-					/* For now, we don't support large POST with truncated data */
-					LM_ERR("got a truncated POST request\n");
-					return MHD_NO;
-				}
-				LM_DBG("got ContentType [%d] with len [%d]: %.*s\\n",
+				LM_DBG("got ContentType [%d] with len [%d]: %.*s\n",
 					pr->content_type, pr->content_len,
 					(int)*upload_data_size, upload_data);
 				/* Here we save data. */
@@ -641,7 +677,7 @@ send_response:
 							(void*)async_data,
 							NULL);
 	} else {
-		return -1;
+		return MHD_NO;
 	}
 
 	if (cb && cb->type>0) {
@@ -654,6 +690,12 @@ send_response:
 		else if (cb->type==HTTPD_TEXT_HTML_TYPE)
 			MHD_add_response_header(response, MHD_HTTP_HEADER_CONTENT_TYPE,
 				"text/html");
+		else if (cb->type==HTTPD_TEXT_PLAIN_TYPE)
+			MHD_add_response_header(response, MHD_HTTP_HEADER_CONTENT_TYPE,
+				"text/plain");
+		else if (cb->type==HTTPD_TEXT_PLAIN_PROMETHEUS_TYPE)
+			MHD_add_response_header(response, MHD_HTTP_HEADER_CONTENT_TYPE,
+				"text/plain; version=0.0.4");
 		else
 			LM_BUG("unhandled content type %d\n",cb->type);
 	} else {
@@ -668,15 +710,28 @@ send_response:
 }
 #endif
 
+
+#ifdef LIBMICROHTTPD
+static fd_set mhd_rs;
+static fd_set mhd_ws;
+static fd_set mhd_es;
+
+int httpd_callback(int fd, void *dmn, int was_timeout)
+{
+	int status;
+	status = MHD_run_from_select((struct MHD_Daemon *)dmn,
+		&mhd_rs, &mhd_ws, &mhd_es);
+	if (status == MHD_NO) {
+		LM_ERR("failed to run http daemon\n");
+		return -1;
+	}
+	return 0;
+}
+#endif
+
+
 void httpd_proc(int rank)
 {
-#ifdef LIBMICROHTTPD
-	int status;
-	fd_set rs;
-	fd_set ws;
-	fd_set es;
-	int max;
-#endif
 	struct httpd_cb *cb = httpd_cb_list;
 
 	/*child's initial settings*/
@@ -699,30 +754,128 @@ void httpd_proc(int rank)
 	}
 
 #ifdef LIBMICROHTTPD
-	struct timeval tv;
-	struct sockaddr_in saddr_in;
+	unsigned int mhd_flags = MHD_USE_DEBUG;
+	unsigned int mhdi_flags;
+	int mhd_opt_n = 0;
+	char *key_pem, *cert_pem, *ip_repr, ip6buf[1+IP_ADDR_MAX_STR_SIZE+1];
+	void *saddr;
+	struct sockaddr_in saddr4;
+	struct MHD_OptionItem mhd_opts[4];
+	const union MHD_DaemonInfo *dmni;
+	int fd;
 
-	memset(&saddr_in, 0, sizeof(saddr_in));
-	if (ip.s)
-		saddr_in.sin_addr.s_addr = inet_addr(ip.s);
-	else
-		saddr_in.sin_addr.s_addr = INADDR_ANY;
-	saddr_in.sin_family = AF_INET;
-	saddr_in.sin_port = htons(port);
+	if (tls_key_file.s && tls_cert_file.s) {
 
+		key_pem = load_file(tls_key_file.s);
+		if (NULL == key_pem) {
+			LM_ERR("unable to load tls key\n");
+			return;
+		}
 
-#if ( MHD_VERSION < 0x000092800 )
-	memcpy( &httpd_server_info, &saddr_in, sizeof(struct sockaddr_in) );
+  		cert_pem = load_file(tls_cert_file.s);
+		if (NULL == cert_pem) {
+			free(key_pem);	
+			LM_ERR("unable to load tls certificate\n");
+			return;
+		}
+
+		mhd_flags = mhd_flags | MHD_USE_SSL;
+
+		mhd_opts[mhd_opt_n].option = MHD_OPTION_HTTPS_MEM_KEY;
+		mhd_opts[mhd_opt_n].value = 0;
+		mhd_opts[mhd_opt_n].ptr_value = key_pem;
+		mhd_opt_n++;
+		mhd_opts[mhd_opt_n].option = MHD_OPTION_HTTPS_MEM_CERT;
+		mhd_opts[mhd_opt_n].value = 0;
+		mhd_opts[mhd_opt_n].ptr_value = cert_pem;
+		mhd_opt_n++;
+		mhd_opts[mhd_opt_n].option = MHD_OPTION_HTTPS_PRIORITIES;
+		mhd_opts[mhd_opt_n].ptr_value = tls_ciphers.s;
+		mhd_opts[mhd_opt_n].value = 0;
+		mhd_opt_n++;
+
+	} else
+		mhd_flags = mhd_flags | MHD_NO_FLAG;
+
+	mhd_opts[mhd_opt_n].option = MHD_OPTION_END;
+	mhd_opts[mhd_opt_n].value = 0;
+	mhd_opts[mhd_opt_n].ptr_value = NULL;
+
+	#if (MHD_VERSION <= 0x00095000)
+	mhd_flags = mhd_flags | MHD_USE_EPOLL_LINUX_ONLY;
+	#else
+	mhd_flags = mhd_flags | MHD_USE_EPOLL;
+	#endif
+
+#if ( MHD_VERSION >= 0x000092800 )
+	struct sockaddr_in6 saddr6;
+	if (ip.s && strcmp(ip.s, "*")) {
+		if (q_memchr(ip.s, ':', ip.len)) {
+			LM_DBG("preparing to listen on IPv6 interface '%s'\n", ip.s);
+			memset(&saddr6, 0, sizeof saddr6);
+
+			if (inet_pton(AF_INET6, ip.s, &saddr6.sin6_addr) <= 0) {
+				LM_ERR("failed to parse 'ip' modparam: %s\n", ip.s);
+				return;
+			}
+
+			saddr6.sin6_family = AF_INET6;
+			saddr6.sin6_port = htons(port);
+			saddr = &saddr6;
+			sprintf(ip6buf, "[%s]", !strcmp(ip.s, "::0") ? "::" : ip.s);
+			ip_repr = ip6buf;
+			mhd_flags |= MHD_USE_IPv6;
+		} else {
+			LM_DBG("preparing to listen on IPv4 interface '%s'\n", ip.s);
+			memset(&saddr4, 0, sizeof saddr4);
+
+			if (inet_pton(AF_INET, ip.s, &saddr4.sin_addr) <= 0) {
+				LM_ERR("failed to parse 'ip' modparam: %s\n", ip.s);
+				return;
+			}
+
+			saddr4.sin_family = AF_INET;
+			saddr4.sin_port = htons(port);
+			saddr = &saddr4;
+			ip_repr = ip.s;
+		}
+	} else {
+		LM_DBG("preparing to listen on all IPv6 + IPv4 interfaces\n");
+		memset(&saddr6, 0, sizeof saddr6);
+
+		saddr6.sin6_addr = in6addr_any;
+		saddr6.sin6_family = AF_INET6;
+		saddr6.sin6_port = htons(port);
+		saddr = &saddr6;
+		ip_repr = "*";
+
+		mhd_flags |= MHD_USE_DUAL_STACK;
+	}
+#else
+	memset(&saddr4, 0, sizeof saddr4);
+	saddr4.sin_family = AF_INET;
+	saddr4.sin_port = htons(port);
+	saddr = &saddr4;
+
+	if (ip.s) {
+		saddr4.sin_addr.s_addr = inet_addr(ip.s);
+		ip_repr = ip.s;
+	} else {
+		saddr4.sin_addr.s_addr = INADDR_ANY;
+		ip_repr = "0.0.0.0";
+	}
+
+	memcpy( &httpd_server_info, &saddr, sizeof(struct sockaddr_in) );
 	httpd_server_info.sin.sin_port = port;
 #endif
 
-
 	LM_DBG("init_child [%d] - [%d] HTTP Server init [%s:%d]\n",
-		rank, getpid(), (ip.s?ip.s:"INADDR_ANY"), port);
-	set_proc_attrs("HTTPD %s:%d", (ip.s?ip.s:"INADDR_ANY"), port);
-	dmn = MHD_start_daemon(MHD_NO_FLAG|MHD_USE_DEBUG, port, NULL, NULL,
+		rank, getpid(), ip_repr, port);
+	set_proc_attrs("HTTPD %s:%d", ip_repr, port);
+	dmn = MHD_start_daemon(mhd_flags, port, NULL, NULL,
 			&(answer_to_connection), NULL,
-			MHD_OPTION_SOCK_ADDR, &saddr_in,
+			MHD_OPTION_SOCK_ADDR, saddr,
+			MHD_OPTION_ARRAY, mhd_opts,
 			MHD_OPTION_END);
 
 	if (NULL == dmn) {
@@ -730,40 +883,49 @@ void httpd_proc(int rank)
 		return;
 	}
 
-	while(1) {
-		max = 0;
-		FD_ZERO (&rs);
-		FD_ZERO (&ws);
-		FD_ZERO (&es);
-		if (MHD_YES != MHD_get_fdset (dmn, &rs, &ws, &es, &max)) {
-			LM_ERR("unable to get file descriptors\n");
-			return;
-		}
-		tv.tv_sec = 1;
-		tv.tv_usec = 0;
-		//LM_DBG("select(%d,%p,%p,%p,%p)\n",max+1, &rs, &ws, &es, &tv);
-		status = select(max+1, &rs, &ws, &es, &tv);
-		if (status < 0) {
-			switch(errno){
-				case EINTR:
-					LM_DBG("error returned by select:"
-							" [%d] [%d][%s]\n",
-							status, errno, strerror(errno));
-					break;
-				default:
-					LM_WARN("error returned by select:"
-							" [%d] [%d][%s]\n",
-							status, errno, strerror(errno));
-					return;
-			}
-		}
-		//LM_DBG("select returned %d\n", status);
-		status = MHD_run_from_select(dmn, &rs, &ws, &es);
-		if (status == MHD_NO) {
-			LM_ERR("unable to run http daemon\n");
-			return;
-		}
+	/* as we use the MHD_USE_EPOLL (so libmicrohttp is internally using
+	 * epoll), the MHD_get_fdset() will return only the epoll controll fd.
+	 * This fd never changes and is set only for reading.
+	 * So, we "learn" that fd right now and bypass MHD_get_fdset() on
+	 * further iterations.*/
+
+	FD_ZERO (&mhd_rs);
+	FD_ZERO (&mhd_ws);
+	FD_ZERO (&mhd_es);
+	MHD_get_fdset (dmn, &mhd_rs, &mhd_ws, &mhd_es, NULL);
+
+#if (MHD_VERSION <= 0x00095100)
+	mhdi_flags = MHD_DAEMON_INFO_EPOLL_FD_LINUX_ONLY;
+#else
+	mhdi_flags = MHD_DAEMON_INFO_EPOLL_FD;
+#endif
+
+	dmni = MHD_get_daemon_info(dmn, mhdi_flags);
+	if (!dmni) {
+		LM_ERR("unable to get file descriptors\n");
+		return;
 	}
+#if (MHD_VERSION <= 0x00095200)
+	fd = dmni->listen_fd;
+#else
+	fd = dmni->epoll_fd;
+#endif
+
+	LM_DBG("found [%d] epoll ctl fd\n",fd);
+
+	if (reactor_proc_init( "HTTPD" )<0) {
+		LM_ERR("failed to init the HTTPD reactor\n");
+		return;
+	}
+
+	if (reactor_proc_add_fd( fd, httpd_callback, (void*)dmn)<0) {
+		LM_CRIT("failed to add Datagram listen socket to reactor\n");
+		return;
+	}
+
+	reactor_proc_loop();
+
+	/* we get here only if the "loop"-ing failed to start*/
 #endif
 	LM_DBG("HTTP Server stopped!\n");
 }
@@ -776,3 +938,33 @@ void httpd_proc_destroy(void)
 #endif
 	return;
 }
+
+static char * load_file(char * filename)
+{
+	FILE *f = fopen(filename, "rb");
+	if(!f)
+		return NULL;
+
+	fseek(f, 0, SEEK_END);
+	long fsize = ftell(f);
+
+	if(fsize <= 0) {
+		fclose(f);
+		return NULL;
+	}
+
+	fseek(f, 0, SEEK_SET);
+
+	char *string = malloc(fsize + 1);
+	size_t bread;
+
+	bread = fread(string, 1, fsize, f);
+	if (bread == 0 || ferror(f))
+		LM_ERR("error while reading from %s (bytes read: %lu)\n",
+				filename, (unsigned long)bread);
+	fclose(f);
+
+	string[fsize] = 0;
+
+	return string;
+}
