@@ -125,7 +125,7 @@ static X509_STORE * store = NULL;
 /* needed for certificate verification */
 static X509_STORE_CTX * verify_ctx = NULL;
 
-static cmd_export_t cmds[]={
+static const cmd_export_t cmds[]={
 	{"authservice",(cmd_function)authservice_, {{0,0,0}},
 		REQUEST_ROUTE | BRANCH_ROUTE | LOCAL_ROUTE},
 	{"verifier",(cmd_function)verifier_, {{0,0,0}},
@@ -133,7 +133,7 @@ static cmd_export_t cmds[]={
 	{0,0,{{0,0,0}},0}
 };
 
-static param_export_t params[]={
+static const param_export_t params[]={
 	{"authCert", STR_PARAM, &authCert},
 	{"privKey", STR_PARAM, &privKey},
 	{"certUri", STR_PARAM, &certUri},
@@ -505,6 +505,7 @@ static int addDate(char * dateHF, time_t * dateHFValue, struct sip_msg * msg)
 	char* buf;
 	size_t len = 0;
 	struct tm * bd_time = NULL;
+	struct tm bd_time_buff;
 
 	if(!dateHF || !dateHFValue || !msg)
 	{
@@ -514,7 +515,7 @@ static int addDate(char * dateHF, time_t * dateHFValue, struct sip_msg * msg)
 
 	*dateHFValue = time(0);
 
-	bd_time = gmtime(dateHFValue);
+	bd_time = gmtime_r(dateHFValue, &bd_time_buff);
 	if(!bd_time)
 	{
 		LM_ERR("gmtime failed\n");
@@ -1301,6 +1302,11 @@ static int checkSign(X509 * cert, char * identityHF, struct sip_msg * msg)
 	/* EVP_DecodeBlock counts the terminating '=', but this padding character does not
 	belong to the signature.*/
 	p=strstr(identityHF , "=");
+	if (!p) {
+		pkg_free(sigbuf);
+		LM_ERR("String '=' not found\n");
+		return 0;
+	}
 	siglen-=strspn(p , "=");
 
 #if OPENSSL_VERSION_NUMBER >= 0x10100000L
@@ -1406,6 +1412,8 @@ static time_t parseX509Date(ASN1_STRING * dateString)
 	unsigned char * tmp = NULL;
 	struct tm tmDate;
 
+	tmDate.tm_isdst = 0;
+
 	if(!dateString)
 	{
 		LM_ERR("dateString not set\n");
