@@ -339,6 +339,7 @@ static int proto_tls_init(struct proto_info *pi)
 	pi->default_port		= tls_port_no;
 
 	pi->tran.init_listener	= proto_tls_init_listener;
+	pi->tran.bind_listener	= tcp_bind_listener;
 	pi->tran.send			= proto_tls_send;
 	pi->tran.dst_attr		= tcp_conn_fcntl;
 
@@ -618,10 +619,13 @@ send_it:
 	send_sock->last_real_ports->local = c->rcv.dst_port;
 	send_sock->last_real_ports->remote = c->rcv.src_port;
 
-	tcp_conn_release(c, 0);
+	tcp_conn_release(c, (rlen<len)?1:0);
 	return rlen;
 con_release:
 	sh_log(c->hist, TCP_SEND2MAIN, "send 1, (%d)", c->refcnt);
+	/* close the fd if this process is not meant to own it */
+	if (c->proc_id != process_no)
+		close(fd);
 	tcp_conn_release(c, (rlen < 0)?0:1);
 	return rlen;
 }
@@ -649,13 +653,6 @@ static int tls_read_req(struct tcp_connection* con, int* bytes_read)
 
 	/* do this trick in order to trace whether if it's an error or not */
 	ret=tls_mgm_api.tls_fix_read_conn(con, con->fd, tls_handshake_tout, t_dst, 1);
-	if (ret < 0) {
-		LM_ERR("failed to do pre-tls handshake!\n");
-		return -1;
-	} else if (ret == 0) {
-		LM_DBG("SSL accept/connect still pending!\n");
-		return 0;
-	}
 
 	/* if there is pending tracing data on an accepted connection, flush it
 	 * As this is a read op, we look only for accepted conns, not to conflict
@@ -675,9 +672,12 @@ static int tls_read_req(struct tcp_connection* con, int* bytes_read)
 		con->proto_flags &= ~( F_TLS_TRACE_READY );
 	}
 
-	if ( ret != 1 ) {
-		LM_ERR("failed to do pre-tls reading\n");
+	if (ret < 0) {
+		LM_ERR("failed to do pre-tls handshake!\n");
 		goto error;
+	} else if (ret == 0) {
+		LM_DBG("SSL accept/connect still pending!\n");
+		return 0;
 	}
 
 	if(con->state!=S_CONN_OK)
