@@ -36,6 +36,7 @@
 #include "pua.h"
 #include "send_publish.h"
 #include "../presence/hash.h"
+#include "clustering.h"
 
 /* database colums */
 static str str_pres_uri_col = str_init("pres_uri");
@@ -78,6 +79,8 @@ void print_ua_pres(ua_pres_t* p)
 	else
 		LM_DBG("expires=[%d] desired_expires=[%d]\n",
 				p->expires, p->desired_expires);
+	if (p->sh_tag.s)
+		LM_DBG("sharing_tag=[%.*s]\n", p->sh_tag.len, p->sh_tag.s);
 }
 
 htable_t* new_htable(void)
@@ -113,7 +116,7 @@ htable_t* new_htable(void)
 			LM_ERR("No more share memory\n");
 			goto error;
 		}
-		H->p_records[i].entity->next= NULL;
+		memset(H->p_records[i].entity, 0, sizeof(ua_pres_t));
 	}
 	return H;
 
@@ -147,7 +150,7 @@ ua_pres_t* search_htable(ua_pres_t* pres, unsigned int hash_code)
 	LM_DBG("\n");
 	for(p= L->next; p; p=p->next)
 	{
-		LM_DBG("Found\n");
+		LM_DBG("Checking\n");
 		print_ua_pres(p);
 		LM_DBG("\n");
 		if((p->flag & pres->flag) && (p->event & pres->event))
@@ -300,6 +303,8 @@ ua_pres_t* new_ua_pres(publ_info_t* publ, str* tuple_id)
 		size+= sizeof(str)+ publ->outbound_proxy.len;
 	if(tuple_id->s)
 		size+= tuple_id->len;
+	if (is_pua_cluster_enabled())
+		size += pua_sh_tag.len;
 
 	presentity= (ua_pres_t*)shm_malloc(size);
 	if(presentity== NULL)
@@ -344,6 +349,13 @@ ua_pres_t* new_ua_pres(publ_info_t* publ, str* tuple_id)
 		size+= publ->outbound_proxy.len;
 	}
 
+	if (is_pua_cluster_enabled()) {
+		presentity->sh_tag.s = (char*)presentity + size;
+		memcpy(presentity->sh_tag.s, pua_sh_tag.s, pua_sh_tag.len);
+		presentity->sh_tag.len = pua_sh_tag.len;
+		size += pua_sh_tag.len;
+	}
+
 	presentity->desired_expires= publ->expires + (int)time(NULL);
 	presentity->flag  = publ->source_flag;
 	presentity->event = publ->event;
@@ -370,10 +382,10 @@ unsigned long new_publ_record(publ_info_t* publ, pua_event_t* ev, str* tuple_id)
 	}
 
 	LM_DBG("cb_param = %p\n", publ->cb_param);
-	return insert_htable(presentity);
+	return insert_htable(presentity, 0);
 }
 
-unsigned long insert_htable(ua_pres_t* presentity)
+unsigned long insert_htable(ua_pres_t* presentity, int mem_only)
 {
 	unsigned int hash_code;
 	str* s1;
@@ -398,17 +410,17 @@ unsigned long insert_htable(ua_pres_t* presentity)
 
 	p= HashT->p_records[hash_code].entity;
 
-	presentity->db_flag= INSERTDB_FLAG;
+	presentity->db_flag= mem_only ? NO_UPDATEDB_FLAG : INSERTDB_FLAG;
 	presentity->next= p->next;
-	if(p->next)
-	{
-		presentity->local_index = p->next->local_index + 1;
-	}
-	else
-		presentity->local_index = 0;
-
 	p->next= presentity;
 
+	p->local_index++;
+	/* local_index cannot be larger than half long, to be sure it will
+	 * not overflow the pres_id */
+	if (p->local_index == (((unsigned long)1)<<(sizeof(long)*8/2))-1)
+		p->local_index = 0;
+	presentity->local_index = p->local_index;
+
 	pres_id = PRES_HASH_ID(presentity);
 
 	lock_release(&HashT->p_records[hash_code].lock);
@@ -416,6 +428,7 @@ unsigned long insert_htable(ua_pres_t* presentity)
 	return pres_id;
 }
 
+
 static void pua_db_delete(ua_pres_t* pres)
 {
 	db_key_t cols[6];
@@ -496,10 +509,11 @@ static void pua_db_delete(ua_pres_t* pres)
 }
 
 
-void free_htable_entry(ua_pres_t* p)
+static void free_htable_entry(ua_pres_t* p, int mem_only)
 {
-	/* delete from database also */
-	pua_db_delete(p);
+	if (!mem_only)
+		/* delete from database also */
+		pua_db_delete(p);
 
 	if(p->etag.s)
 		shm_free(p->etag.s);
@@ -520,7 +534,7 @@ void delete_htable_safe(ua_pres_t* p, unsigned int hash_index)
 
 	if(q)
 		q->next = p->next;
-	free_htable_entry(p);
+	free_htable_entry(p,0);
 }
 
 
@@ -536,7 +550,7 @@ void delete_htable(unsigned int hash_index, unsigned int local_index)
 		if(p->local_index == local_index)
 		{
 			q->next = p->next;
-			free_htable_entry(p);
+			free_htable_entry(p,0);
 			break;
 		}
 		q = p;
@@ -544,6 +558,7 @@ void delete_htable(unsigned int hash_index, unsigned int local_index)
 	lock_release(&HashT->p_records[hash_index].lock);
 }
 
+
 void destroy_htable(void)
 {
 	ua_pres_t* p= NULL,*q= NULL;
@@ -574,6 +589,60 @@ void destroy_htable(void)
 	return;
 }
 
+
+int get_record_coordinates(ua_pres_t* pres, unsigned int *hash_code,
+		unsigned int *label_index)
+{
+	ua_pres_t *presentity;
+
+	*hash_code = core_hash( pres->pres_uri, NULL, HASH_SIZE);
+
+	lock_get(&HashT->p_records[*hash_code].lock);
+
+	presentity = search_htable(pres, *hash_code);
+
+	if (presentity==NULL) {
+		lock_release(&HashT->p_records[*hash_code].lock);
+		return -1;
+	}
+
+	*label_index = presentity->local_index;
+
+	lock_release(&HashT->p_records[*hash_code].lock);
+
+	return 0;
+}
+
+
+/* replaces the in-hash presentity with coordinates (hash_index,local_index)
+ * with a new presentity record */
+int replace_in_htable(unsigned int hash_index, unsigned int local_index,
+		ua_pres_t *new_pres)
+{
+	ua_pres_t* p= NULL, *q= NULL;
+
+	lock_get(&HashT->p_records[hash_index].lock);
+
+	q = HashT->p_records[hash_index].entity;
+	for(p= q->next; p; p=p->next)
+	{
+		if(p->local_index == local_index)
+		{
+			q->next = p->next;
+			q->next = new_pres;
+			new_pres->next = p->next;
+			new_pres->local_index = local_index;
+			free_htable_entry(p,1/*mem only, no db*/);
+			lock_release(&HashT->p_records[hash_index].lock);
+			return 0;
+		}
+		q = p;
+	}
+	lock_release(&HashT->p_records[hash_index].lock);
+	return -1;
+}
+
+
 /* must lock the record line before calling this function*/
 ua_pres_t* get_dialog(ua_pres_t* dialog, unsigned int hash_code)
 {
