@@ -22,6 +22,8 @@
  * History:
  * --------
  *  2009-01-14  initial version (Iñaki Baz Castillo)
+ *  2023-08-12  export pcres_match to MI (Fabien Aunay)
+ *  2023-08-12  export pcres_match_group to MI (Fabien Aunay)
  */
 
 
@@ -47,8 +49,6 @@
 #include "../../mod_fix.h"
 #include "../../mi/mi.h"
 
-
-
 #define START 0
 #define RELOAD 1
 
@@ -108,14 +108,15 @@ static int w_pcre_match_group(struct sip_msg* _msg, str* string, int* _num_pcre)
 /*
  * MI functions
  */
-mi_response_t *mi_pcres_reload(const mi_params_t *params,
-								struct mi_handler *async_hdl);
+mi_response_t *mi_pcres_reload(const mi_params_t *params, struct mi_handler *async_hdl);
+mi_response_t *mi_pcres_match(const mi_params_t *params, struct mi_handler *async_hdl);
+mi_response_t *mi_pcres_match_group(const mi_params_t *params, struct mi_handler *async_hdl);
 
 
 /*
  * Exported functions
  */
-static cmd_export_t cmds[] =
+static const cmd_export_t cmds[] =
 {
 	{"pcre_match", (cmd_function)w_pcre_match, {
 		{CMD_PARAM_STR,0,0},
@@ -133,7 +134,7 @@ static cmd_export_t cmds[] =
 /*
  * Exported parameters
  */
-static param_export_t params[] = {
+static const param_export_t params[] = {
 	{"file",                STR_PARAM,  &file                },
 	{"max_groups",          INT_PARAM,  &max_groups          },
 	{"group_max_size",      INT_PARAM,  &group_max_size      },
@@ -148,9 +149,17 @@ static param_export_t params[] = {
 /*
  * Exported MI functions
  */
-static mi_export_t mi_cmds[] = {
-	{ "regex_reload", 0, 0, 0, {
-		{mi_pcres_reload, {0}},
+static const mi_export_t mi_cmds[] = {
+	{ "regex_reload", "Causes regex module to re-read the content of the text file and re-compile the regular expressions", 0, 0, 
+		{{mi_pcres_reload, {0}},
+		{EMPTY_MI_RECIPE}}
+	},
+	{ "regex_match", "Matches the given string parameter against the regular expression pcre_regex", 0, 0, {
+		{mi_pcres_match, {"string", "pcre_regex", 0}},
+		{EMPTY_MI_RECIPE}}
+	},
+	{ "regex_match_group", "It uses the groups readed from the text file to match the given string parameter against the compiled regular expression in group number group", 0, 0, {
+		{mi_pcres_match_group, {"string", "group", 0}},
 		{EMPTY_MI_RECIPE}}
 	},
 	{EMPTY_MI_EXPORT}
@@ -183,7 +192,6 @@ struct module_exports exports = {
 };
 
 
-
 /*! \brief
  * Init module function
  */
@@ -326,7 +334,8 @@ static int load_pcres(int action)
 			i++;
 			/* Check if there are more patterns than the max value */
 			if (i >= max_groups) {
-				LM_ERR("max patterns exceeded\n");
+				i--;
+				LM_ERR("max_groups: %d exceeded\n",max_groups);
 				fclose(f);
 				goto err;
 			}
@@ -653,8 +662,7 @@ static int w_pcre_match_group(struct sip_msg* _msg, str* string, int* _num_pcre)
  */
 
 /*! \brief Reload pcres by reading the file again */
-mi_response_t *mi_pcres_reload(const mi_params_t *params,
-								struct mi_handler *async_hdl)
+mi_response_t *mi_pcres_reload(const mi_params_t *params, struct mi_handler *async_hdl)
 {
 	/* Check if group matching feature is enabled */
 	if (file == NULL) {
@@ -667,6 +675,102 @@ mi_response_t *mi_pcres_reload(const mi_params_t *params,
 		LM_ERR("failed to reload pcres\n");
 		return init_mi_error(500, MI_SSTR("Internal error"));
 	}
+
 	LM_NOTICE("reload success\n");
 	return init_mi_result_ok();
 }
+
+
+/*! \brief Matches the given string parameter against the regular expression pcre_regex */
+mi_response_t *mi_pcres_match(const mi_params_t *params, struct mi_handler *async_hdl)
+{
+	str string, pcre_regex;
+	int rc;
+
+	if ( get_mi_string_param(params, "string", &string.s, &string.len ) < 0) {
+		LM_DBG("mi_pcres_match string param error\n");
+		return init_mi_param_error();
+	}
+	if ( get_mi_string_param(params, "pcre_regex", &pcre_regex.s, &pcre_regex.len) < 0) {
+		LM_DBG("mi_pcres_match pcre_regex param error\n");
+		return init_mi_param_error();
+	}
+
+	/* handle call back function result */
+	rc = w_pcre_match(NULL, &string, &pcre_regex);
+	LM_DBG("w_pcre_match: string<%s>, pcre_regex=<%s>, result:<%i>\n", string.s, pcre_regex.s, rc);
+	
+	switch(rc) {
+		case -4:
+			/* Compilation error */
+			return init_mi_error(500, MI_SSTR("Error pcre_re compilation"));
+			break;
+		case -1:
+			/* Not Match */
+			return init_mi_result_string(MI_SSTR("Not Match"));
+			break;
+		case 1:
+			/* Match */
+			return init_mi_result_string(MI_SSTR("Match"));
+			break;
+		default:
+			/* Any other case */
+			return init_mi_error(500, MI_SSTR("Error"));
+	}
+}
+
+/*! \brief It uses the groups readed from the text file to match the given string parameter against
+ * the compiled regular expression in group number group
+ */
+mi_response_t *mi_pcres_match_group(const mi_params_t *params, struct mi_handler *async_hdl)
+{
+	str string, group;
+	int _group;
+	int rc;
+
+	if ( get_mi_string_param(params, "string", &string.s, &string.len ) < 0) {
+		LM_DBG("mi_pcres_match_group string param error\n");
+		return init_mi_param_error();
+	}
+	if ( get_mi_string_param(params, "group", &group.s, &group.len) < 0) {
+		LM_DBG("mi_pcres_match_group group param error\n");
+		return init_mi_param_error();
+	}
+
+	/* 
+	 *	type casting MI Param -> int(group) function.
+	 *	L.616 already test if group is an integer, if not, default 0 is set.
+	 */
+	_group = atoi(group.s);
+
+	/* No possible negative index */
+	if ( _group < 0 ) {
+		return init_mi_error(500, MI_SSTR("Error invalid pcre index"));
+	}
+
+	/* handle call back function result */
+	rc = w_pcre_match_group(NULL, &string, &_group);
+	LM_DBG("w_pcre_match_group: string<%s>, _group=<%i>, result:<%i>\n", string.s, _group, rc);
+
+	switch(rc) {
+		case -4:
+			/* Compilation error */
+			return init_mi_error(500, MI_SSTR("Error invalid pcre index"));
+			break;
+		case -2:
+			/* group is disabled */
+			return init_mi_error(500, MI_SSTR("Error group matching is disabled"));
+			break;
+		case -1:
+			/* Not Match */
+			return init_mi_result_string(MI_SSTR("Not Match"));
+			break;
+		case 1:
+			/* Match */
+			return init_mi_result_string(MI_SSTR("Match"));
+			break;
+		default:
+			/* Any other case */
+			return init_mi_error(500, MI_SSTR("Error"));
+	}
+}
