@@ -31,7 +31,7 @@
 
 int sl_startup();
 int sl_shutdown();
-int sl_send_reply( struct sip_msg *msg, int code, str *reason, str *totag);
+int sl_send_reply( struct sip_msg *msg, int code, const str *reason, str *totag);
 int sl_filter_ACK( struct sip_msg *msg, void *foo);
 int sl_reply_error( struct sip_msg *msg );
 int sl_gen_totag( struct sip_msg *msg, str *totag);
