@@ -66,7 +66,7 @@
 
 #define lump_clone( _new,_old,_ptr) \
 	{\
-		(_new) = (struct lump*)(_ptr);\
+		(_new) = (struct lump*)(void *)(_ptr);\
 		memcpy( (_new), (_old), sizeof(struct lump) );\
 		(_new)->flags|=LUMPFLAG_SHMEM; \
 		(_ptr)+=ROUND4(sizeof(struct lump));\
@@ -92,7 +92,7 @@ inline static struct via_body* via_body_cloner( char* new_buf,
 	do
 	{
 		/* clones the via_body structure */
-		new_via = (struct via_body*)(*p);
+		new_via = (struct via_body*)(void *)(*p);
 		memcpy( new_via , org_via , sizeof( struct via_body) );
 		(*p) += ROUND4(sizeof( struct via_body ));
 
@@ -125,7 +125,7 @@ inline static struct via_body* via_body_cloner( char* new_buf,
 			struct via_param *vp, *new_vp, *last_new_vp;
 			for( vp=org_via->param_lst, last_new_vp=0 ; vp ; vp=vp->next )
 			{
-				new_vp = (struct via_param*)(*p);
+				new_vp = (struct via_param*)(void *)(*p);
 				memcpy( new_vp , vp , sizeof(struct via_param));
 				(*p) += ROUND4(sizeof(struct via_param));
 				new_vp->name.s=translate_pointer(new_buf,org_buf,vp->name.s);
@@ -217,7 +217,7 @@ static inline struct auth_body* auth_body_cloner(char* new_buf, char *org_buf, s
 {
 	struct auth_body* new_auth;
 
-	new_auth = (struct auth_body*)(*p);
+	new_auth = (struct auth_body*)(void *)(*p);
 	memcpy(new_auth , auth , sizeof(struct auth_body));
 	(*p) += ROUND4(sizeof(struct auth_body));
 
@@ -378,7 +378,7 @@ do { \
 		struct lump_rpl   *_rpl_lump, **_rpl_lump_anchor; \
 		_rpl_lump_anchor = (_anchor); \
 		for(_rpl_lump=(_list);_rpl_lump;_rpl_lump=_rpl_lump->next) { \
-			*(_rpl_lump_anchor)=(struct lump_rpl*)(_p); \
+			*(_rpl_lump_anchor)=(struct lump_rpl*)(void *)(_p); \
 			(_p) += ROUND4(sizeof( struct lump_rpl )); \
 			(*_rpl_lump_anchor)->flags = LUMP_RPL_SHMEM | \
 				(_rpl_lump->flags&(~(LUMP_RPL_NODUP|LUMP_RPL_NOFREE))); \
@@ -487,19 +487,47 @@ struct sip_msg*  sip_msg_cloner( struct sip_msg *org_msg, int *sip_msg_len,
 			case HDR_PPI_T:
 			case HDR_PAI_T:
 			case HDR_PRIVACY_T:
-			case HDR_RETRY_AFTER_T:
 			case HDR_CALL_INFO_T:
 			case HDR_WWW_AUTHENTICATE_T:
 			case HDR_PROXY_AUTHENTICATE_T:
+			case HDR_MIN_EXPIRES_T:
 			case HDR_FEATURE_CAPS_T:
+			case HDR_REPLACES_T:
+			case HDR_SECURITY_CLIENT_T:
+			case HDR_SECURITY_SERVER_T:
+			case HDR_SECURITY_VERIFY_T:
 				/* we ignore them for now even if they have something parsed*/
 				break;
 
+			/* not used in sip_msg, used in MSRP */
+			case HDR_TO_PATH_T:
+			case HDR_FROM_PATH_T:
+			case HDR_MESSAGE_ID_T:
+			case HDR_BYTE_RANGE_T:
+			case HDR_FAILURE_REPORT_T:
+			case HDR_SUCCESS_REPORT_T:
+			case HDR_STATUS_T:
+			case HDR_USE_PATH_T:
+
+			/* not having shortcut */
+			case HDR_RETRY_AFTER_T:
+			case HDR_VIA2_T:
+
+			/* not actual hdrs */
+			case HDR_OTHER_T:
+			case HDR_ERROR_T:
+			case HDR_EOH_T:
+				break;
+
+			/* we do not have a "default" on purpose, so we get
+			 * a compile err/war where a new HDR is added and we do 
+			 * not handle it here.
 			default:
 				if (hdr->parsed) {
 					LM_WARN("header body ignored: %d\n", hdr->type );
 				}
 				break;
+			*/
 		}/*switch*/
 	}/*for all headers*/
 
@@ -541,13 +569,14 @@ struct sip_msg*  sip_msg_cloner( struct sip_msg *org_msg, int *sip_msg_len,
 		*sip_msg_len = len;
 
 	/* filling up the new structure */
-	new_msg = (struct sip_msg*)p;
+	new_msg = (struct sip_msg*)(void *)p;
 	/* sip msg structure */
 	memcpy( new_msg , org_msg , sizeof(struct sip_msg));
 
 	/* avoid copying pointer to un-clonned structures */
 	new_msg->body = NULL;
 	new_msg->msg_cb = NULL;
+	new_msg->sdp_ops = NULL;
 
 	new_msg->msg_flags |= FL_SHM_CLONE;
 	p += ROUND4(sizeof(struct sip_msg));
@@ -600,7 +629,7 @@ struct sip_msg*  sip_msg_cloner( struct sip_msg *org_msg, int *sip_msg_len,
 	new_msg->via2=0;
 	for( hdr=org_msg->headers,last_hdr=0 ; hdr ; hdr=hdr->next )
 	{
-		new_hdr = (struct hdr_field*)p;
+		new_hdr = (struct hdr_field*)(void *)p;
 		memcpy(new_hdr, hdr, sizeof(struct hdr_field) );
 		p += ROUND4(sizeof( struct hdr_field));
 		new_hdr->name.s = translate_pointer(new_msg->buf, org_msg->buf,
@@ -636,7 +665,6 @@ struct sip_msg*  sip_msg_cloner( struct sip_msg *org_msg, int *sip_msg_len,
 				else if ( !new_msg->via2 )
 				{
 					LINK_SIBLING_HEADER(h_via1, new_hdr);
-					new_msg->h_via2 = new_hdr;
 					new_msg->via2 = via_body_cloner( new_msg->buf,
 						org_msg->buf, (struct via_body*)hdr->parsed, &p);
 					new_hdr->parsed  = (void*)new_msg->via2;
@@ -697,7 +725,7 @@ struct sip_msg*  sip_msg_cloner( struct sip_msg *org_msg, int *sip_msg_len,
 				for(;to_prm;to_prm=to_prm->next)
 				{
 					/*alloc*/
-					new_to_prm = (struct to_param*)p;
+					new_to_prm = (struct to_param*)(void *)p;
 					p +=ROUND4(sizeof(struct to_param ));
 					/*coping*/
 					memcpy( new_to_prm, to_prm, sizeof(struct to_param ));
@@ -935,6 +963,39 @@ struct sip_msg*  sip_msg_cloner( struct sip_msg *org_msg, int *sip_msg_len,
 					new_msg->proxy_authenticate = new_hdr;
 				}
 				break;
+			case HDR_MIN_EXPIRES_T:
+				if (HOOK_NOT_SET(min_expires)) {
+					new_msg->min_expires = new_hdr;
+				}
+				break;
+			case HDR_FEATURE_CAPS_T:
+				if (HOOK_NOT_SET(feature_caps)) {
+					new_msg->feature_caps = new_hdr;
+				}
+				break;
+			case HDR_REPLACES_T:
+				if (HOOK_NOT_SET(replaces)) {
+					new_msg->replaces = new_hdr;
+				}
+				break;
+			case HDR_SECURITY_CLIENT_T:
+				if (HOOK_NOT_SET(security_client))
+					new_msg->security_client = new_hdr;
+				else
+					LINK_SIBLING_HEADER(security_client, new_hdr);
+				break;
+			case HDR_SECURITY_SERVER_T:
+				if (HOOK_NOT_SET(security_server))
+					new_msg->security_server = new_hdr;
+				else
+					LINK_SIBLING_HEADER(security_server, new_hdr);
+				break;
+			case HDR_SECURITY_VERIFY_T:
+				if (HOOK_NOT_SET(security_verify))
+					new_msg->security_verify = new_hdr;
+				else
+					LINK_SIBLING_HEADER(security_verify, new_hdr);
+				break;
 			default:
 				/* ignore the rest*/
 				;
